package SeMeet::Helpers::Core;

use base 'Mojolicious::Plugin';
use feature 'signatures';
no warnings qw{ experimental::signatures } ;

use YAML;
use Mojo::JWT;
use Mojo::JSON qw(encode_json);

sub register  {
    my ($class, $self ) = @_;

    $self->helper( trace => sub($c, $data, @args) {

        if ( ref $data && ref $data eq 'SCALAR') {
            $data = sprintf(${ $data }, @args);
        }
        elsif ( ref $data ) {
            $data = Dump $data;
        }

        $c->app->log->debug($data);
    });

    $self->helper(error => sub ($c, $status, $msg) {

        if ( ref $msg eq 'ARRAY' ) {
            $errors = $msg;
        }
        elsif ( ref $msg eq 'HASH' ) {
            $errors = [ $msg ];
        }
        else {
            $errors = [{
                message => $c->l($msg),
                path    => $c->stash('openapi.path'),
            }];
        }

        $c->stash(
            status  => $status,
            openapi => {
                status  => $status,
                errors => $errors,
            },
        );

        return undef;
    });

    $self->helper( jwt_token => sub {
        my $c     = shift;
        my $token = shift;

        my $jwt;

        eval {
            $jwt = Mojo::JWT->new(
                secret => $c->cfg->{jwt_secret}
            )->decode($token);
        };

        if ( $@ ) {
            $c->app->log->warn( $@ );
            return 'Invalid token';
        }

        if ( Mojo::JWT->now() > $jwt->{exp} ) {
            $c->app->log->warn( $@ );
            return 'Token expire'
        }

        return $jwt;

    });

    $self->helper( spec_filter => sub ($c, $data, $class) {

        if (my $def =$c->openapi->spec("/components/schemas/$class")) {
            my $filtered = {};

            KEY:
            foreach my $key ( keys %{ $def->{properties} } ) {
                my $value    = $data->{$key};
                my $nullable = 0;

                my $types = $def->{properties}{$key}{type};

                if ( ref $types eq 'ARRAY' ) {
                    TYPE:
                    foreach my $type ( @{ $types } ) {
                        $nullable = 1, last if $type eq 'null';
                    }
                }

                $filtered->{$key} = $value;
            }

            $data = $filtered;

        }

        return $data;
    });

    $self->helper( user_by_uuid => sub ($c, $uuid) {
        my $user = $c->schema->resultset('User')->find({
            uuid => $uuid
        });

        if ( ! $user ) {
            my $octopus_user = $c->iapi->get("octopus/users/$uuid");

            if ( $octopus_user ) {
                $user = $c->schema->resultset('User')->find_or_create(
                    {
                        uuid        => $uuid,
                        username    => $octopus_user->{username},
                        displayname => $octopus_user->{displayname},
                    },
                    { key => 'uuid' }
                );
            }
        }

        return $user;

    });
}

1;

